/* 
 *	Copyright (C) 2004-2005 Vadim Berezniker
 *	http://www.kryptolus.com
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

#include "stdafx.h"
#include "common.h"

#include "sabbu.h"

#include "kryArray.h"
#include "kryString.h"
#include "krySSACommandParser.h"
#include "krySSACommandHighlighter.h"

#include "gui_main_style.h"
#include "gui_main_script.h"
#include "gui_text_editor.h"
#include "gui_status_bar.h"
#include "gui_event_list.h"
#include "sound.h"

extern struct sabbu app;

enum color_columns
{
  COLUMN_COLOR_NAME,
  COLUMN_COLOR_COLOR,
  COLUMN_COLOR_POINTER
};

struct _gui_customize_colors
{
  struct KryTextEditor *text_editor;
  struct KryStatusBar *status_bars[3];

  GtkButton *button_reset_section;
} gui_customize_colors;

void gui_customize_colors_fill_list(kryColorTable *color_table, GtkTreeStore *store, GtkTreeIter *iter_parent)
{
  int actual_count = 0;
  for(int i = 0; i < color_table->GetColorCount(); i++)
  {
    kryColor *color = color_table->Get(i);
    if(!color->IsInitialized())
      continue;
    
    actual_count++;
  }

  int cur_color = 0;
  for(int i = 0; i < color_table->GetColorCount(); i++)
  {
    kryColor *color = color_table->Get(i);
    if(!color->IsInitialized())
      continue;

    GtkTreeIter iter;
    gtk_tree_store_append(store, &iter, iter_parent);

    gtk_tree_store_set(store, &iter, 
      COLUMN_COLOR_NAME, color->GetName(), 
      COLUMN_COLOR_POINTER, color,
    -1);

    cur_color++;
  }
}

void gui_customize_colors_selection_changed_cb(GtkTreeSelection *selection, GtkFrame *preview_frame)
{
  GtkTreeModel *model;
  GList *rows = gtk_tree_selection_get_selected_rows(selection, &model);
  if(!rows)
    return;

  GtkTreePath *path = (GtkTreePath *) rows->data;
  GtkTreeIter iter;
  kryColor *color;
  char *text;

  gtk_tree_model_get_iter(model, &iter, path);
  gtk_tree_model_get(model, &iter,
    COLUMN_COLOR_POINTER, &color, 
    COLUMN_COLOR_NAME, &text,
    -1);

  if(gtk_tree_path_get_depth(path) > 1)
  {
    kry_free(text);
    gtk_widget_show(GTK_WIDGET(app.ui.tab_options_color.hbox_editor));
    gtk_widget_hide(GTK_WIDGET(app.ui.tab_options_color.label_no_color));

    app.ui.tab_options_color.active_color = color;
    gtk_widget_queue_draw(GTK_WIDGET(app.ui.tab_options_color.area));

    GtkVBox *vbox = GTK_VBOX(gtk_bin_get_child(GTK_BIN(preview_frame)));
    GtkWidget *widget_preview = NULL;
    GtkTreeIter iter_parent;
    if(gtk_tree_model_iter_parent(model, &iter_parent, &iter))
    {
      gtk_tree_model_get(model, &iter_parent, 
        COLUMN_COLOR_POINTER, &widget_preview, 
        COLUMN_COLOR_NAME, &text,
        -1);
      if(widget_preview)
      {
        gtk_widget_show(widget_preview);
        gtk_widget_show(GTK_WIDGET(preview_frame));
      }
      else
      {
        gtk_widget_hide(GTK_WIDGET(preview_frame));
      }
    }

    GList *children = gtk_container_get_children(GTK_CONTAINER(vbox));
    for(GList *ptr = children; ptr; ptr = ptr->next)
      if(ptr->data != widget_preview)
        gtk_widget_hide(GTK_WIDGET(ptr->data));
  }
  else
  {
    app.ui.tab_options_color.active_color = NULL;
    gtk_widget_hide(GTK_WIDGET(app.ui.tab_options_color.hbox_editor));
    gtk_widget_show(GTK_WIDGET(app.ui.tab_options_color.label_no_color));

    gtk_widget_hide(GTK_WIDGET(preview_frame));
  }

  char *section = kry_strdup_printf(KRY_LOC _("Reset '%s' Colors"), text);
  gtk_button_set_label(gui_customize_colors.button_reset_section, section);
  kry_free(section);
  kry_free(text);

  g_list_foreach(rows, (GFunc) (gtk_tree_path_free), NULL);
  g_list_free(rows);
}

void gui_customize_colors_expose_cb(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
  if(!app.ui.tab_options_color.active_color)
    return;

  gui_main_color_fill(GTK_DRAWING_AREA(widget), app.ui.tab_options_color.active_color, TRUE);
}

void gui_customize_colors_modify_cb(GtkWidget *widget, GtkWindow *window)
{
  GtkColorSelectionDialog *dialog = GTK_COLOR_SELECTION_DIALOG(gtk_color_selection_dialog_new(_("Modify Color")));

  kryColor *color = app.ui.tab_options_color.active_color;
  GdkColor color_gdk;
  color_gdk.red = color->GetRed() * 257;
  color_gdk.green = color->GetGreen() * 257;
  color_gdk.blue = color->GetBlue() * 257;
  gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(dialog->colorsel), &color_gdk);

  gtk_widget_set_sensitive(GTK_WIDGET(window), FALSE);
  int rv = gtk_dialog_run(GTK_DIALOG(dialog));
  if(rv == GTK_RESPONSE_OK)
  {
    gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(dialog->colorsel), &color_gdk);
    color->SetColor(color_gdk.red / 257, color_gdk.green / 257, color_gdk.blue / 257);

    char *val = kry_strdup_printf(KRY_LOC  "%d", color->GetRed() << 16 | color->GetGreen() << 8 | color->GetBlue());
    if(color->GetTextID())
      app.prefs->SetString("Colors", color->GetTextID(), val);
    kry_free(val);
   }

  gtk_widget_destroy(GTK_WIDGET(dialog));

  gtk_widget_set_sensitive(GTK_WIDGET(window), TRUE);

  return;
}

void gui_customize_colors_reset_cb(GtkWidget *widget, gpointer data)
{
  app.ui.tab_options_color.active_color->Reset();
  gtk_widget_queue_draw(GTK_WIDGET(app.ui.tab_options_color.area));
  gtk_widget_queue_draw(GTK_WIDGET(app.ui.tab_options_color.color_view));
}

void gui_customize_colors_reset_all_recur(GtkTreeModel *model, GtkTreeIter iter)
{
  for(;;)
  {
    kryColor *color = NULL;
    GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
    if(gtk_tree_path_get_depth(path) > 1)
    {
      gtk_tree_model_get(model, &iter, COLUMN_COLOR_POINTER, &color, -1);
      if(color)
        color->Reset();
    }


    if(gtk_tree_model_iter_has_child(model, &iter))
    {
      GtkTreeIter iter_child;
      gtk_tree_model_iter_children(model, &iter_child, &iter);
      gui_customize_colors_reset_all_recur(model, iter_child);
    }

    if(!gtk_tree_model_iter_next(model, &iter))
      break;
  }
}

void gui_customize_colors_reset_section_cb(GtkWidget *widget, GtkTreeView *view)
{
  GtkTreeIter iter_child, iter;
  GtkTreeModel *model;
  GtkTreeSelection *selection = gtk_tree_view_get_selection(view);
  GList *sel_items = gtk_tree_selection_get_selected_rows(selection, &model);
  if(!sel_items)
    return;

  GtkTreePath *path = (GtkTreePath *) sel_items->data;
  
  gtk_tree_model_get_iter(model, &iter_child, path);
  if(!gtk_tree_model_iter_parent(model, &iter, &iter_child))
    iter = iter_child;
  
  if(gtk_tree_model_iter_children(model, &iter_child, &iter))
    gui_customize_colors_reset_all_recur(model, iter_child);

  gtk_widget_queue_draw(GTK_WIDGET(app.ui.tab_options_color.area));
  gtk_widget_queue_draw(GTK_WIDGET(app.ui.tab_options_color.color_view));
}

void gui_customize_colors_reset_all_cb(GtkWidget *widget, GtkTreeStore *store)
{
  GtkTreeIter iter;
  if(!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter))
    return;

  gui_customize_colors_reset_all_recur(GTK_TREE_MODEL(store), iter);

  gtk_widget_queue_draw(GTK_WIDGET(app.ui.tab_options_color.area));
  gtk_widget_queue_draw(GTK_WIDGET(app.ui.tab_options_color.color_view));
}

void gui_customize_colors_color_column_cb(GtkTreeViewColumn *column, GtkCellRenderer *renderer, 
                                    GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
{
  kryColor *color;
  gtk_tree_model_get(model, iter, COLUMN_COLOR_POINTER, &color, -1);

  GtkTreePath *path = gtk_tree_model_get_path(model, iter);
  if(gtk_tree_path_get_depth(path) <= 1)
  {
    gtk_tree_path_free(path);
    return;
  }

  gtk_tree_path_free(path);

  char *str = kry_strdup_printf(KRY_LOC "#%02X%02X%02X", color->GetRed(), color->GetGreen(), color->GetBlue());
  g_object_set(renderer, "background", str, NULL);
  kry_free(str);
}

void gui_customize_colors_free()
{
  gui_text_editor_free(gui_customize_colors.text_editor);
  gui_status_bar_free(gui_customize_colors.status_bars[0]);
  gui_status_bar_free(gui_customize_colors.status_bars[1]);
  gui_status_bar_free(gui_customize_colors.status_bars[2]);
}

void gui_customize_colors_close_cb(GtkWidget *widget, GtkWindow *window)
{
  gui_customize_colors_free();
  gui_main_enable();
  gui_main_focus();
  gtk_widget_destroy(GTK_WIDGET(window));
}

gboolean gui_customize_colors_delete_cb(GtkWidget *widget, gpointer data)
{
  gui_customize_colors_free();
  gui_main_enable();
  gui_main_focus();
  return FALSE;
}

void gui_customize_colors_show()
{
  GtkWindow *window = GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
  GtkScrolledWindow *sw = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
  GtkTreeStore *color_store = gtk_tree_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
  GtkTreeView *color_view = GTK_TREE_VIEW(gtk_tree_view_new_with_model(GTK_TREE_MODEL(color_store)));
  GtkCellRendererText *renderer_name = GTK_CELL_RENDERER_TEXT(gtk_cell_renderer_text_new());
  GtkCellRendererText *renderer_color = GTK_CELL_RENDERER_TEXT(gtk_cell_renderer_text_new());
  GtkTreeViewColumn *column_name = gtk_tree_view_column_new_with_attributes(__("ColorList|Name"), GTK_CELL_RENDERER(renderer_name),
    "text", COLUMN_COLOR_NAME,
    NULL);
  GtkTreeViewColumn *column_color = gtk_tree_view_column_new_with_attributes(__("ColorList|Color"), GTK_CELL_RENDERER(renderer_color),
    "background", COLUMN_COLOR_COLOR,
    NULL);
  GtkTreeSelection *selection = gtk_tree_view_get_selection(color_view);

  gtk_window_set_title(window, _("Customize Colors"));

  gtk_tree_view_column_set_cell_data_func(column_color, GTK_CELL_RENDERER(renderer_color), gui_customize_colors_color_column_cb, NULL, NULL);

  gtk_tree_view_set_headers_visible(color_view, FALSE);
  gtk_tree_view_column_set_sizing(column_color, GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width(column_color, 40);

  gtk_tree_view_append_column(color_view, column_name);
  gtk_tree_view_append_column(color_view, column_color);

  gtk_scrolled_window_set_policy(sw, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(sw), GTK_WIDGET(color_view));

  GtkHBox *hbox_colors = GTK_HBOX(gtk_hbox_new(FALSE, 0));

  app.ui.tab_options_color.hbox_editor = hbox_colors;
  app.ui.tab_options_color.label_no_color = GTK_LABEL(gtk_label_new(_("Please select a color on the left")));
  app.ui.tab_options_color.color_view = color_view;

  gtk_widget_hide(GTK_WIDGET(app.ui.tab_options_color.label_no_color));


  GtkFrame *notice = GTK_FRAME(gtk_frame_new(NULL));
  GtkVBox *vbox_colors = GTK_VBOX(gtk_vbox_new(FALSE, 0));
  gtk_container_add(GTK_CONTAINER(notice), gtk_label_new(_("All changes take effect instantly")));
  gtk_box_pack_start(GTK_BOX(vbox_colors), GTK_WIDGET(notice), FALSE, TRUE, 0);

  gtk_container_set_border_width(GTK_CONTAINER(notice), 5);


  {
    GtkFrame *frame_edit = GTK_FRAME(gtk_frame_new(__("ColorOptionns|Edit Color")));
    GtkDrawingArea *area = GTK_DRAWING_AREA(gtk_drawing_area_new());

    GtkHBox *hbox_color_edit = GTK_HBOX(gtk_hbox_new(FALSE, 0));
    GtkHBox *hbox_color_frame = GTK_HBOX(gtk_hbox_new(FALSE, 0));
    
    GtkLabel *label_color = GTK_LABEL(gtk_label_new("Color:"));
    GtkButton *button_modify = GTK_BUTTON(gtk_button_new_with_label("Modify"));
    GtkButton *button_reset = GTK_BUTTON(gtk_button_new_with_label("Reset"));
    GtkButton *button_reset_section = GTK_BUTTON(gtk_button_new());
    GtkButton *button_reset_all = GTK_BUTTON(gtk_button_new_with_label("Reset All"));

    gui_customize_colors.button_reset_section = button_reset_section;

    gtk_box_pack_start(GTK_BOX(hbox_color_frame), GTK_WIDGET(app.ui.tab_options_color.label_no_color), TRUE, TRUE, 3);
    gtk_box_pack_start(GTK_BOX(hbox_color_edit), GTK_WIDGET(label_color), FALSE, TRUE, 1);
    gtk_box_pack_start(GTK_BOX(hbox_color_edit), GTK_WIDGET(area), FALSE, TRUE, 1);
    gtk_box_pack_start(GTK_BOX(hbox_color_edit), GTK_WIDGET(button_modify), FALSE, TRUE, 1);
    gtk_box_pack_start(GTK_BOX(hbox_color_edit), GTK_WIDGET(button_reset), FALSE, TRUE, 1);

    gtk_widget_set_size_request(GTK_WIDGET(area), 40, -1);
    g_signal_connect(G_OBJECT(area), "expose_event", G_CALLBACK(gui_customize_colors_expose_cb), NULL);
    g_signal_connect(G_OBJECT(button_modify), "clicked", G_CALLBACK(gui_customize_colors_modify_cb), window);
    g_signal_connect(G_OBJECT(button_reset), "clicked", G_CALLBACK(gui_customize_colors_reset_cb), NULL);
    g_signal_connect(G_OBJECT(button_reset_section), "clicked", G_CALLBACK(gui_customize_colors_reset_section_cb), color_view);
    g_signal_connect(G_OBJECT(button_reset_all), "clicked", G_CALLBACK(gui_customize_colors_reset_all_cb), color_store);

    gtk_box_pack_start(GTK_BOX(hbox_color_frame), GTK_WIDGET(hbox_color_edit), TRUE, TRUE, 0);

    gtk_box_pack_start(GTK_BOX(hbox_color_frame), GTK_WIDGET(gtk_label_new(NULL)), TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_color_frame), GTK_WIDGET(button_reset_section), FALSE, TRUE, 1);
    gtk_box_pack_start(GTK_BOX(hbox_color_frame), GTK_WIDGET(button_reset_all), FALSE, TRUE, 1);


    gtk_container_set_border_width(GTK_CONTAINER(hbox_color_frame), 3);
    gtk_container_add(GTK_CONTAINER(frame_edit), GTK_WIDGET(hbox_color_frame));

    gtk_box_pack_start(GTK_BOX(vbox_colors), GTK_WIDGET(frame_edit), FALSE, TRUE, 0);

    app.ui.tab_options_color.hbox_editor = hbox_color_edit;
    app.ui.tab_options_color.area = area;
  }

  GtkFrame *frame_preview = GTK_FRAME(gtk_frame_new(__("ColorOptions|Preview")));
  KryWaveformGroup *waveform_group = KRY_WAVEFORM_GROUP(kry_waveform_group_new());
  GtkTreeView *event_list_view;
  GtkWidget *text_editor;
  GtkVBox *vbox_status_bars;
  struct KryStatusBar **bars = gui_customize_colors.status_bars;
  // preview frame
  {

    GtkVBox *vbox_preview = GTK_VBOX(gtk_vbox_new(FALSE, 0));
    gtk_container_set_border_width(GTK_CONTAINER(vbox_preview), 2);
    
    // waveform
    {
      gtk_container_add(GTK_CONTAINER(frame_preview), GTK_WIDGET(vbox_preview));

      struct KryWaveformDataInfo *data_info = kry_new0(struct KryWaveformDataInfo);
      data_info->waveform_data = (char *) kry_malloc(5 * 2000);
      data_info->max_pos = 255;
      data_info->max_neg = 255;
      data_info->vals_per_sec = 1000;
      data_info->length_seconds = 5;


      int wavelength = 500;
      for(int i = 0; i < 5 * 2000; i++)
      {
        data_info->waveform_data[i] = (char) (sin((M_PI * ((i % wavelength) / (double) wavelength))) * 200);
      }

      KryWaveform *waveform = KRY_WAVEFORM(kry_waveform_new(data_info, kry_waveform_group_get_adjustment(waveform_group), app.ui.waveform_color_table));
      KryWaveformGroupItem *item = KRY_WAVEFORM_GROUP_ITEM(kry_waveform_group_item_new(waveform));
      GtkWidget *scrollbar = gtk_hscrollbar_new(kry_waveform_group_get_adjustment(waveform_group));

      kry_waveform_group_item_set_scrollbar(item, scrollbar);

      kry_waveform_group_waveform_add(waveform_group, item);
      kry_waveform_group_set_zoom(waveform_group, 1.0);

      gtk_box_pack_start(GTK_BOX(vbox_preview), GTK_WIDGET(waveform_group), FALSE, TRUE, 2);
      gtk_box_pack_start(GTK_BOX(vbox_colors), GTK_WIDGET(frame_preview), FALSE, TRUE, 3);

      app.ui.wave_preview = waveform;

      KryMarker *marker_start = KRY_MARKER(kry_marker_new(MARKER_START, 1));
      kry_marker_set_value(marker_start, 100);
      kry_waveform_group_marker_add(waveform_group, marker_start);

      KryMarker *marker_end = KRY_MARKER(kry_marker_new(MARKER_END, 1));
      kry_marker_set_value(marker_end, 200);
      kry_waveform_group_marker_add(waveform_group, marker_end);

      kry_waveform_marker_set_active_start(waveform, marker_start);
      kry_waveform_marker_set_active_end(waveform, marker_end);
      kry_waveform_group_region_add_new(waveform_group, marker_start, marker_end, WAVEFORM_COLOR_SELECTION_HIGHLIGHT);

      KryMarker *marker_current = KRY_MARKER(kry_marker_new(MARKER_CURRENT, 1));
      kry_marker_set_value(marker_current, 150);
      kry_waveform_group_marker_add(waveform_group, marker_current);

      struct KryWaveformKeyframeList *list = kry_new(struct KryWaveformKeyframeList, 1);
      long *keyframes = kry_new(long, 5);
      keyframes[0] = 269;
      keyframes[1] = 413;
      keyframes[2] = 456;
      keyframes[3] = 690;
      keyframes[4] = 805;

      list->list = keyframes;
      list->count = 5;

      kry_waveform_set_keyframe_list(waveform, list);

      kry_waveform_group_set_drag_selection(waveform_group, app.opts.enable_drag_selection);
      kry_waveform_group_set_marker_dragging(waveform_group, app.opts.enable_marker_dragging);
    }

    // event list

    {
      struct KryEventList *event_list = gui_event_list_new(TRUE);

      gui_event_list_update_visible_columns(event_list, app.ui.ui_mode);

      event_list_view = gui_event_list_get_view(event_list);

      kryList<kryEventDetailed *> list;
      kryEventDetailed *event1 = new kryEventDetailed();
      event1->SetType(kryEvent::EVENT_COMMENT_DIALOG);
      event1->SetText("Comment");
      event1->SetName("Toshi");
      event1->SetTime(15250, 21100);
      event1->SetIndex(0);

      kryEventDetailed *event2 = new kryEventDetailed();
      event2->SetType(kryEvent::EVENT_DIALOG);
      event2->SetText("Regular");
      event2->SetName("Liu");
      event2->SetTime(21500, 23100);
      event2->SetIndex(1);

      kryEventDetailed *event3 = new kryEventDetailed();
      event3->SetType(kryEvent::EVENT_DIALOG);
      event3->SetText("Collision");
      event3->SetName("Vadim");
      event3->SetTime(24100, 26350);
      event3->SetCollisionAdded(TRUE);
      event3->SetIndex(2);

      kryEventDetailed *event4 = new kryEventDetailed();
      event4->SetType(kryEvent::EVENT_DIALOG);
      event4->SetText("Highlighted Collision");
      event4->SetName("Henry");
      event4->SetTime(25120, 27800);
      event4->SetCollisionHighlighted(TRUE);
      event4->SetIndex(3);

      kryEventDetailed *event5 = new kryEventDetailed();
      event5->SetType(kryEvent::EVENT_DIALOG);
      event5->SetText("Regular");
      event5->SetName("Sumi");
      event5->SetTime(28250, 31700);
      event5->SetIndex(4);

      list.Append(event1);
      list.Append(event2);
      list.Append(event3);
      list.Append(event4);
      list.Append(event5);

      gui_event_list_fill_from_list_detailed(event_list, &list, FALSE);

      //gui_event_list_set_row_color(event_list, 0, COLOR

      gtk_box_pack_start(GTK_BOX(vbox_preview), GTK_WIDGET(gui_event_list_get_view(event_list)), FALSE, TRUE, 2);
    }

    // text editor
    {
      char *text = 
        "{\\r\\t(0,1000,\\3c&HFFFFFF&\\fscx200)}s"
        "{\\r\\t(1000,2000,\\3c&HFFFFFF&\\fscx200)}a"
        "{\\r\\t(2000,3000,\\3c&HFFFFFF&\\fscx200)}b"
        "{\\r\\t(3000,4000,\\3c&HFFFFFF&\\fscx200)}b"
        "{\\r\\t(4000,5000,\\3c&HFFFFFF&\\fscx200)}u{pretty}!";

      struct KryTextEditor *editor = gui_text_editor_new();
      gui_customize_colors.text_editor = editor;

      gui_text_editor_set_text(editor, text);
      gui_text_editor_set_syntax_highlight_mode(editor, SYNTAX_HIGHLIGHT_SYNTAX);

      text_editor = gui_text_editor_get_widget(editor);

      gtk_box_pack_start(GTK_BOX(vbox_preview), text_editor, FALSE, TRUE, 2);
    }

    // status bar
    {
      vbox_status_bars = GTK_VBOX(gtk_vbox_new(FALSE, 0));
      
      bars[0] = gui_status_bar_new();
      gui_status_bar_set_text(bars[0], STATUS_WELCOME, _("Welcome to Sabbu"));
      gtk_box_pack_start(GTK_BOX(vbox_status_bars), gui_status_bar_get_widget(bars[0]), FALSE, TRUE, 2);

      bars[1] = gui_status_bar_new();
      gui_status_bar_set_text_with_color(bars[1], STATUS_MODIFIED, 
        _("There are UNSAVED Changes"), app.ui.status_bar_color_table->Get(STATUS_COLOR_UNSAVED));
      gtk_box_pack_start(GTK_BOX(vbox_status_bars), gui_status_bar_get_widget(bars[1]), FALSE, TRUE, 2);

      bars[2] = gui_status_bar_new();
      gui_status_bar_set_text_with_color(bars[2], STATUS_SAVED, 
        __("Status|Script Saved"), app.ui.status_bar_color_table->Get(STATUS_COLOR_SUCCESS));
      gtk_box_pack_start(GTK_BOX(vbox_status_bars), gui_status_bar_get_widget(bars[2]), FALSE, TRUE, 2);

      gtk_box_pack_start(GTK_BOX(vbox_preview), GTK_WIDGET(vbox_status_bars), FALSE, TRUE, 2);
    }
    
  }

  g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(gui_customize_colors_selection_changed_cb), frame_preview);
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);

  {
    GtkTreeIter iter;
    gtk_tree_store_append(color_store, &iter, NULL);
    gtk_tree_store_set(color_store, &iter, COLUMN_COLOR_POINTER, event_list_view, -1);
    gtk_tree_store_set(color_store, &iter, COLUMN_COLOR_NAME, __("ColorOptions|Event List"), -1);
    gui_customize_colors_fill_list(app.ui.list_color_table, color_store, &iter);

    gtk_tree_store_append(color_store, &iter, NULL);
    gtk_tree_store_set(color_store, &iter, COLUMN_COLOR_POINTER, waveform_group, -1);
    gtk_tree_store_set(color_store, &iter, COLUMN_COLOR_NAME, __("ColorOptions|Waveform"), -1);
    gui_customize_colors_fill_list(app.ui.waveform_color_table, color_store, &iter);

    gtk_tree_store_append(color_store, &iter, NULL);
    gtk_tree_store_set(color_store, &iter, COLUMN_COLOR_POINTER, vbox_status_bars, -1);
    gtk_tree_store_set(color_store, &iter, COLUMN_COLOR_NAME, __("ColorOptions|Status Bar"), -1);
    gui_customize_colors_fill_list(app.ui.status_bar_color_table, color_store, &iter);

    gtk_tree_store_append(color_store, &iter, NULL);
    gtk_tree_store_set(color_store, &iter, COLUMN_COLOR_POINTER, text_editor, -1);
    gtk_tree_store_set(color_store, &iter, COLUMN_COLOR_NAME, __("ColorOptions|Syntax Highlighting"), -1);
    gui_customize_colors_fill_list(app.ui.highlight_color_table, color_store, &iter);

    GtkFrame *frame_colors = GTK_FRAME(gtk_frame_new(NULL));
    gtk_container_add(GTK_CONTAINER(frame_colors), GTK_WIDGET(sw));
    gtk_box_pack_start(GTK_BOX(hbox_colors), GTK_WIDGET(frame_colors), FALSE, TRUE, 3);
  }
  gtk_box_pack_start(GTK_BOX(hbox_colors), GTK_WIDGET(vbox_colors), TRUE, TRUE, 3);
  gtk_tree_view_expand_all(color_view);

  GtkHButtonBox *bbox = GTK_HBUTTON_BOX(gtk_hbutton_box_new());
  GtkButton *button_close = GTK_BUTTON(gtk_button_new_from_stock("gtk-close"));
  GtkHBox *hbox_button = GTK_HBOX(gtk_hbox_new(FALSE, 0));
  gtk_box_pack_start(GTK_BOX(bbox), GTK_WIDGET(button_close), FALSE, TRUE, 0);
  gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
  gtk_box_pack_start(GTK_BOX(hbox_button), GTK_WIDGET(bbox), TRUE, TRUE, 7);

  g_signal_connect(G_OBJECT(button_close), "clicked", G_CALLBACK(gui_customize_colors_close_cb), window);
  g_signal_connect(G_OBJECT(window), "delete-event", G_CALLBACK(gui_customize_colors_delete_cb), NULL);

  gtk_box_pack_start(GTK_BOX(vbox_colors), gtk_label_new(NULL), TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(vbox_colors), GTK_WIDGET(hbox_button), FALSE, FALSE, 5);


  gtk_container_add(GTK_CONTAINER(window), GTK_WIDGET(hbox_colors));
  gtk_window_set_transient_for(window, app.ui.window);
  gtk_window_set_position(window, GTK_WIN_POS_CENTER_ON_PARENT);

  gtk_widget_set_size_request(GTK_WIDGET(window), -1, 400);

  gui_main_disable();
  gtk_widget_show_all(GTK_WIDGET(window));

  gui_status_bar_hide_progress_bar(bars[0]);
  gui_status_bar_hide_progress_bar(bars[1]);
  gui_status_bar_hide_progress_bar(bars[2]);
}


