/* 
 *	Copyright (C) 2004-2005 Vadim Berezniker
 *	http://www.kryptolus.com
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

#include "stdafx.h"

#include "common.h"

#include "sabbu.h"

#include "gui_export.h"
#include "gui_export_encore.h"

extern struct sabbu app;

void gui_export_encore_strip_cb(GtkToggleButton *widget, struct export_settings *settings)
{
  settings->strip_ssa_commands = gtk_toggle_button_get_active(widget);
}

void gui_export_encore_check_cb(GtkToggleButton *widget, gboolean *val)
{
  *val = gtk_toggle_button_get_active(widget);
}

struct export_settings_encore *gui_export_encore_show()
{
  GtkDialog *dialog = GTK_DIALOG(gtk_dialog_new_with_buttons(_("Adobe Encore Export Settings"), app.ui.window, 
    GTK_DIALOG_MODAL, GTK_STOCK_OK, GTK_RESPONSE_OK, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
    NULL));
  GtkVBox *vbox = GTK_VBOX(gtk_vbox_new(FALSE, 0));
  GtkComboBoxEntry *combo_fps;
  struct export_settings_encore *settings = kry_new0(struct export_settings_encore);

  app.prefs->PushSection("ExportAdobeEncore");
  // frame rate frame
  {
    GtkEventBox *border = GTK_EVENT_BOX(gtk_event_box_new());
    GtkFrame *frame_fps = GTK_FRAME(gtk_frame_new(_("Video Frame Rate")));
    combo_fps = gui_combo_box_fps_new(TRUE);

    char *fps_set = app.prefs->GetString("FrameRate");
    if(!fps_set)
      fps_set = "29.97";

    if(app.ui.tab_video.video)
    {
      double fps = video_get_fps(app.ui.tab_video.video);
      if(fabs(fps - 29.97) < 0.0001)
        fps_set = "29.97";
      else if(fabs(fps - 25) < 0.0001)
        fps_set = "25";
    }

    gui_combo_box_set_text(GTK_COMBO_BOX(combo_fps), fps_set, TRUE);

    gtk_editable_set_editable(GTK_EDITABLE(gtk_bin_get_child(GTK_BIN(combo_fps))), FALSE);

    gtk_container_set_border_width(GTK_CONTAINER(border), 2);

    gtk_container_add(GTK_CONTAINER(border), GTK_WIDGET(combo_fps));
    gtk_container_add(GTK_CONTAINER(frame_fps), GTK_WIDGET(border));
    gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(frame_fps), FALSE, TRUE, 0);
  }

  {
    GtkFrame *frame_options = GTK_FRAME(gtk_frame_new(_("Export Options")));
    GtkVBox *vbox_options = GTK_VBOX(gtk_vbox_new(FALSE, 0));
    GtkCheckButton *button_comments = GTK_CHECK_BUTTON(gtk_check_button_new_with_label(__("ExportText|Comments")));

    g_signal_connect(G_OBJECT(button_comments), "toggled", G_CALLBACK(gui_export_encore_check_cb), &settings->comments);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button_comments), app.prefs->GetInt("Comments", 1));

    gtk_box_pack_start(GTK_BOX(vbox_options), GTK_WIDGET(button_comments), FALSE, TRUE, 2);

    gtk_container_add(GTK_CONTAINER(frame_options), GTK_WIDGET(vbox_options));
    gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(frame_options), FALSE, TRUE, 0);
  }

  gtk_box_pack_start(GTK_BOX(vbox), gui_export_format_options_create_ui(&settings->settings), FALSE, TRUE, 0);
 
  gtk_box_pack_start(GTK_BOX(dialog->vbox), GTK_WIDGET(vbox), FALSE, TRUE, 2);
  gtk_widget_show_all(GTK_WIDGET(dialog));

  int rv = gtk_dialog_run(dialog);

  if(rv == GTK_RESPONSE_OK)
  {
    char *text = KRY_TS((gtk_combo_box_get_active_text(GTK_COMBO_BOX(combo_fps))));
    if(!strcmp(text, "25 (PAL)"))
      settings->fps = ENCORE_FPS_PAL;
    else
      settings->fps = ENCORE_FPS_NTSC;

    app.prefs->SetString("FrameRate", text);
    kry_free(text);

    gui_export_format_options_save(&settings->settings);
  }
  else
  {
    kry_free(settings);
    settings = NULL;
  }

  app.prefs->PopSection();

  gui_main_focus();
  gtk_widget_destroy(GTK_WIDGET(dialog));

  return settings;
}


